@@ -46,9 +46,6 @@ typedef struct _GApplicationClass                           GApplicationClass;
 /**
  * GApplication:
  *
- * The <structname>GApplication</structname> structure contains private
- * data and should only be accessed using the provided API
- *
  * Since: 2.28
  */
 struct _GApplication
@@ -59,35 +56,6 @@ struct _GApplication
   GApplicationPrivate *priv;
 };
 
-/**
- * GApplicationClass:
- * @startup: invoked on the primary instance immediately after registration
- * @shutdown: invoked only on the registered primary instance immediately
- *      after the main loop terminates
- * @activate: invoked on the primary instance when an activation occurs
- * @open: invoked on the primary instance when there are files to open
- * @command_line: invoked on the primary instance when a command-line is
- *   not handled locally
- * @local_command_line: invoked (locally) when the process has been invoked
- *     via commandline execution.  The virtual function has the chance to
- *     inspect (and possibly replace) the list of command line arguments.
- *     See g_application_run() for more information.
- * @before_emit: invoked on the primary instance before 'activate', 'open',
- *     'command-line' or any action invocation, gets the 'platform data' from
- *     the calling instance
- * @after_emit: invoked on the primary instance after 'activate', 'open',
- *     'command-line' or any action invocation, gets the 'platform data' from
- *     the calling instance
- * @add_platform_data: invoked (locally) to add 'platform data' to be sent to
- *     the primary instance when activating, opening or invoking actions
- * @quit_mainloop: invoked on the primary instance when the use count of the
- *     application drops to zero (and after any inactivity timeout, if
- *     requested)
- * @run_mainloop: invoked on the primary instance from g_application_run()
- *     if the use-count is non-zero
- *
- * Since: 2.28
- */
 struct _GApplicationClass
 {
   /*< private >*/
@@ -145,9 +113,18 @@ GApplicationFlags       g_application_get_flags                         (GApplic
 void                    g_application_set_flags                         (GApplication             *application,
                                                                          GApplicationFlags         flags);
 
+GLIB_DEPRECATED
 void                    g_application_set_action_group                  (GApplication             *application,
                                                                          GActionGroup             *action_group);
 
+void                    g_application_set_app_menu                      (GApplication             *application,
+                                                                         GMenuModel               *app_menu);
+GMenuModel             *g_application_get_app_menu                      (GApplication             *application);
+
+void                    g_application_set_menubar                       (GApplication             *application,
+                                                                         GMenuModel               *menubar);
+GMenuModel *            g_application_get_menubar                       (GApplication             *application);
+
 gboolean                g_application_get_is_registered                 (GApplication             *application);
 gboolean                g_application_get_is_remote                     (GApplication             *application);
 
@@ -169,6 +146,9 @@ int                     g_application_run                               (GApplic
                                                                          int                       argc,
                                                                          char                    **argv);
 
+GApplication *          g_application_get_default                       (void);
+void                    g_application_set_default                       (GApplication             *application);
+
 G_END_DECLS
 
 #endif /* __G_APPLICATION_H__ */
