@@ -2,12 +2,10 @@
  * Copyright © 2010 Codethink Limited
  * Copyright © 2011 Canonical Limited
  *
- * SPDX-License-Identifier: LGPL-2.1-or-later
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as published
+ * by the Free Software Foundation; either version 2 of the licence or (at
+ * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,27 +13,29 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * Public License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  * Authors: Ryan Lortie <desrt@desrt.ca>
  */
 
 #include "config.h"
 
-#include "gdbusactiongroup-private.h"
+#include "gdbusactiongroup.h"
 
-#include "gremoteactiongroup.h"
 #include "gdbusconnection.h"
 #include "gactiongroup.h"
 
 /**
- * GDBusActionGroup:
+ * SECTION:gdbusactiongroup
+ * @title: GDBusActionGroup
+ * @short_description: A D-Bus GActionGroup implementation
+ * @see_also: <link linkend="gio-GActionGroup-exporter">GActionGroup exporter</link>
  *
- * `GDBusActionGroup` is an implementation of the [iface@Gio.ActionGroup]
- * interface.
- *
- * `GDBusActionGroup` can be used as a proxy for an action group
- * that is exported over D-Bus with [method@Gio.DBusConnection.export_action_group].
+ * #GDBusActionGroup is an implementation of the #GActionGroup
+ * interface that can be used as a proxy for an action group
+ * that is exported over D-Bus with g_dbus_connection_export_action_group().
  */
 
 struct _GDBusActionGroup
@@ -91,7 +91,7 @@ action_info_free (gpointer user_data)
   g_slice_free (ActionInfo, info);
 }
 
-static ActionInfo *
+ActionInfo *
 action_info_new_from_iter (GVariantIter *iter)
 {
   const gchar *param_str;
@@ -100,7 +100,7 @@ action_info_new_from_iter (GVariantIter *iter)
   GVariant *state;
   gchar *name;
 
-  if (!g_variant_iter_next (iter, "{s(b&g@av)}", &name,
+  if (!g_variant_iter_next (iter, "{s(bg@av)}", &name,
                             &enabled, &param_str, &state))
     return NULL;
 
@@ -122,11 +122,9 @@ action_info_new_from_iter (GVariantIter *iter)
   return info;
 }
 
-static void g_dbus_action_group_remote_iface_init (GRemoteActionGroupInterface *iface);
-static void g_dbus_action_group_iface_init        (GActionGroupInterface       *iface);
+static void g_dbus_action_group_iface_init (GActionGroupInterface *);
 G_DEFINE_TYPE_WITH_CODE (GDBusActionGroup, g_dbus_action_group, G_TYPE_OBJECT,
-  G_IMPLEMENT_INTERFACE (G_TYPE_ACTION_GROUP, g_dbus_action_group_iface_init)
-  G_IMPLEMENT_INTERFACE (G_TYPE_REMOTE_ACTION_GROUP, g_dbus_action_group_remote_iface_init))
+  G_IMPLEMENT_INTERFACE (G_TYPE_ACTION_GROUP, g_dbus_action_group_iface_init))
 
 static void
 g_dbus_action_group_changed (GDBusConnection *connection,
@@ -231,7 +229,6 @@ g_dbus_action_group_changed (GDBusConnection *connection,
             else
               action_info_free (info);
           }
-        g_variant_iter_free (iter);
       }
     }
 }
@@ -267,8 +264,6 @@ g_dbus_action_group_describe_all_done (GObject      *source,
       g_variant_iter_free (iter);
       g_variant_unref (reply);
     }
-
-  g_object_unref (group);
 }
 
 
@@ -284,7 +279,7 @@ g_dbus_action_group_async_init (GDBusActionGroup *group)
 
   g_dbus_connection_call (group->connection, group->bus_name, group->object_path, "org.gtk.Actions", "DescribeAll", NULL,
                           G_VARIANT_TYPE ("(a{s(bgav)})"), G_DBUS_CALL_FLAGS_NONE, -1, NULL,
-                          g_dbus_action_group_describe_all_done, g_object_ref (group));
+                          g_dbus_action_group_describe_all_done, group);
 }
 
 static gchar **
@@ -296,7 +291,7 @@ g_dbus_action_group_list_actions (GActionGroup *g_group)
   if (group->actions != NULL)
     {
       GHashTableIter iter;
-      unsigned int n, i = 0;
+      gint n, i = 0;
       gpointer key;
 
       n = g_hash_table_size (group->actions);
@@ -305,7 +300,7 @@ g_dbus_action_group_list_actions (GActionGroup *g_group)
       g_hash_table_iter_init (&iter, group->actions);
       while (g_hash_table_iter_next (&iter, &key, NULL))
         keys[i++] = g_strdup (key);
-      g_assert (i == n);
+      g_assert_cmpint (i, ==, n);
       keys[n] = NULL;
     }
   else
@@ -361,69 +356,49 @@ g_dbus_action_group_query_action (GActionGroup        *g_group,
   else
     {
       g_dbus_action_group_async_init (group);
-      group->strict = TRUE;
 
       return FALSE;
     }
 }
 
 static void
-g_dbus_action_group_activate_action_full (GRemoteActionGroup *remote,
-                                          const gchar        *action_name,
-                                          GVariant           *parameter,
-                                          GVariant           *platform_data)
+g_dbus_action_group_change_state (GActionGroup *g_group,
+                                  const gchar  *action_name,
+                                  GVariant     *value)
 {
-  GDBusActionGroup *group = G_DBUS_ACTION_GROUP (remote);
+  GDBusActionGroup *group = G_DBUS_ACTION_GROUP (g_group);
+
+  /* Don't bother with the checks.  The other side will do it again. */
+  g_dbus_connection_call (group->connection, group->bus_name, group->object_path, "org.gtk.Actions", "SetState",
+                          g_variant_new ("(sva{sv})", action_name, value, NULL),
+                          NULL, G_DBUS_CALL_FLAGS_NONE, -1, NULL, NULL, NULL);
+}
+
+static void
+g_dbus_action_group_activate (GActionGroup *g_group,
+                              const gchar  *action_name,
+                              GVariant     *parameter)
+{
+  GDBusActionGroup *group = G_DBUS_ACTION_GROUP (g_group);
   GVariantBuilder builder;
 
-  g_variant_builder_init_static (&builder, G_VARIANT_TYPE ("av"));
+  g_variant_builder_init (&builder, G_VARIANT_TYPE ("av"));
 
   if (parameter)
     g_variant_builder_add (&builder, "v", parameter);
 
   g_dbus_connection_call (group->connection, group->bus_name, group->object_path, "org.gtk.Actions", "Activate",
-                          g_variant_new ("(sav@a{sv})", action_name, &builder, platform_data),
+                          g_variant_new ("(sava{sv})", action_name, &builder, NULL),
                           NULL, G_DBUS_CALL_FLAGS_NONE, -1, NULL, NULL, NULL);
 }
 
-static void
-g_dbus_action_group_change_action_state_full (GRemoteActionGroup *remote,
-                                              const gchar        *action_name,
-                                              GVariant           *value,
-                                              GVariant           *platform_data)
-{
-  GDBusActionGroup *group = G_DBUS_ACTION_GROUP (remote);
-
-  g_dbus_connection_call (group->connection, group->bus_name, group->object_path, "org.gtk.Actions", "SetState",
-                          g_variant_new ("(sv@a{sv})", action_name, value, platform_data),
-                          NULL, G_DBUS_CALL_FLAGS_NONE, -1, NULL, NULL, NULL);
-}
-
-static void
-g_dbus_action_group_change_state (GActionGroup *group,
-                                  const gchar  *action_name,
-                                  GVariant     *value)
-{
-  g_dbus_action_group_change_action_state_full (G_REMOTE_ACTION_GROUP (group),
-                                                action_name, value, g_variant_new ("a{sv}", NULL));
-}
-
-static void
-g_dbus_action_group_activate (GActionGroup *group,
-                              const gchar  *action_name,
-                              GVariant     *parameter)
-{
-  g_dbus_action_group_activate_action_full (G_REMOTE_ACTION_GROUP (group),
-                                            action_name, parameter, g_variant_new ("a{sv}", NULL));
-}
-
 static void
 g_dbus_action_group_finalize (GObject *object)
 {
   GDBusActionGroup *group = G_DBUS_ACTION_GROUP (object);
 
   if (group->subscription_id)
-    g_dbus_connection_signal_unsubscribe (group->connection, g_steal_handle_id (&group->subscription_id));
+    g_dbus_connection_signal_unsubscribe (group->connection, group->subscription_id);
 
   if (group->actions)
     g_hash_table_unref (group->actions);
@@ -449,13 +424,6 @@ g_dbus_action_group_class_init (GDBusActionGroupClass *class)
   object_class->finalize = g_dbus_action_group_finalize;
 }
 
-static void
-g_dbus_action_group_remote_iface_init (GRemoteActionGroupInterface *iface)
-{
-  iface->activate_action_full = g_dbus_action_group_activate_action_full;
-  iface->change_action_state_full = g_dbus_action_group_change_action_state_full;
-}
-
 static void
 g_dbus_action_group_iface_init (GActionGroupInterface *iface)
 {
@@ -468,11 +436,10 @@ g_dbus_action_group_iface_init (GActionGroupInterface *iface)
 /**
  * g_dbus_action_group_get:
  * @connection: A #GDBusConnection
- * @bus_name: (nullable): the bus name which exports the action
- *     group or %NULL if @connection is not a message bus connection
+ * @bus_name: the bus name which exports the action group
  * @object_path: the object path at which the action group is exported
  *
- * Obtains a #GDBusActionGroup for the action group which is exported at
+ * Obtains a #GDBusAcitonGroup for the action group which is exported at
  * the given @bus_name and @object_path.
  *
  * The thread default main context is taken at the time of this call.
@@ -497,8 +464,6 @@ g_dbus_action_group_get (GDBusConnection *connection,
 {
   GDBusActionGroup *group;
 
-  g_return_val_if_fail (bus_name != NULL || g_dbus_connection_get_unique_name (connection) == NULL, NULL);
-
   group = g_object_new (G_TYPE_DBUS_ACTION_GROUP, NULL);
   group->connection = g_object_ref (connection);
   group->bus_name = g_strdup (bus_name);
@@ -507,7 +472,7 @@ g_dbus_action_group_get (GDBusConnection *connection,
   return group;
 }
 
-gboolean
+G_GNUC_INTERNAL gboolean
 g_dbus_action_group_sync (GDBusActionGroup  *group,
                           GCancellable      *cancellable,
                           GError           **error)
