@@ -1,12 +1,10 @@
 /*
  * Copyright © 2011 Canonical Ltd.
  *
- * SPDX-License-Identifier: LGPL-2.1-or-later
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * licence, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,26 +12,28 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+ * USA.
  *
  * Author: Ryan Lortie <desrt@desrt.ca>
  */
 
-#include "config.h"
-
 #include "gdbusmenumodel.h"
 
-#include "gmenuexporter.h"
 #include "gmenumodel.h"
 
 /* Prelude {{{1 */
 
 /**
- * GDBusMenuModel:
+ * SECTION:gdbusmenumodel
+ * @title: GDBusMenuModel
+ * @short_description: A D-Bus GMenuModel implementation
+ * @see_also: <link linkend="gio-GMenuModel-exporter">GMenuModel Exporter</link>
  *
- * `GDBusMenuModel` is an implementation of [class@Gio.MenuModel] that can be
- * used as a proxy for a menu model that is exported over D-Bus with
- * [method@Gio.DBusConnection.export_menu_model].
+ * #GDBusMenuModel is an implementation of #GMenuModel that can be used
+ * as a proxy for a menu model that is exported over D-Bus with
+ * g_dbus_connection_export_menu_model().
  */
 
 /*
@@ -195,7 +195,7 @@ path_identifier_equal (gconstpointer a,
   ConstPathIdentifier *id_b = b;
 
   return id_a->connection == id_b->connection &&
-         g_strcmp0 (id_a->bus_name, id_b->bus_name) == 0 &&
+         g_str_equal (id_a->bus_name, id_b->bus_name) &&
          g_str_equal (id_a->object_path, id_b->object_path);
 }
 
@@ -306,7 +306,7 @@ static void
 g_dbus_menu_path_deactivate (GDBusMenuPath *path)
 {
   if (--path->active == 0)
-    g_dbus_connection_signal_unsubscribe (path->id->connection, g_steal_handle_id (&path->watch_id));
+    g_dbus_connection_signal_unsubscribe (path->id->connection, path->watch_id);
 }
 
 static GDBusMenuPath *
@@ -381,7 +381,6 @@ g_dbus_menu_group_unref (GDBusMenuGroup *group)
 
       g_hash_table_remove (group->path->groups, GINT_TO_POINTER (group->id));
       g_hash_table_unref (group->proxies);
-      g_hash_table_unref (group->menus);
 
       g_dbus_menu_path_unref (group->path);
 
@@ -570,8 +569,6 @@ g_dbus_menu_group_deactivate (GDBusMenuGroup *group)
     }
 }
 
-/* @menu_id, @position, @removed and @added are all untrusted since they can
- * come from an external process. */
 static void
 g_dbus_menu_group_changed (GDBusMenuGroup *group,
                            guint           menu_id,
@@ -585,20 +582,6 @@ g_dbus_menu_group_changed (GDBusMenuGroup *group,
   GSequence *items;
   GVariant *item;
   gint n_added;
-  gint n_items;
-
-  /* Caller has to check this. */
-  g_assert (g_variant_is_of_type (added, G_VARIANT_TYPE ("aa{sv}")));
-
-  n_added = g_variant_n_children (added);
-
-  if (position < 0 || position >= G_MENU_EXPORTER_MAX_SECTION_SIZE ||
-      removed < 0 || removed >= G_MENU_EXPORTER_MAX_SECTION_SIZE ||
-      n_added >= G_MENU_EXPORTER_MAX_SECTION_SIZE)
-    {
-      g_warning ("invalid arguments");
-      return;
-    }
 
   /* We could have signals coming to us when we're not active (due to
    * some other process having subscribed to this group) or when we're
@@ -617,18 +600,10 @@ g_dbus_menu_group_changed (GDBusMenuGroup *group,
       g_hash_table_insert (group->menus, GINT_TO_POINTER (menu_id), items);
     }
 
-  /* Don’t need to worry about overflow due to the low value of
-   * %G_MENU_EXPORTER_MAX_SECTION_SIZE. */
-  n_items = g_sequence_get_length (items);
-  if (position + removed > n_items ||
-      n_items - removed + n_added > G_MENU_EXPORTER_MAX_SECTION_SIZE)
-    {
-      g_warning ("invalid arguments");
-      return;
-    }
-
   point = g_sequence_get_iter_at_pos (items, position + removed);
 
+  g_return_if_fail (point != NULL);
+
   if (removed)
     {
       GSequenceIter *start;
@@ -637,11 +612,11 @@ g_dbus_menu_group_changed (GDBusMenuGroup *group,
       g_sequence_remove_range (start, point);
     }
 
-  g_variant_iter_init (&iter, added);
+  n_added = g_variant_iter_init (&iter, added);
   while (g_variant_iter_loop (&iter, "@a{sv}", &item))
     g_sequence_insert_before (point, g_dbus_menu_group_create_item (item));
 
-  if (g_sequence_is_empty (items))
+  if (g_sequence_get_length (items) == 0)
     {
       g_hash_table_remove (group->menus, GINT_TO_POINTER (menu_id));
       items = NULL;
@@ -873,8 +848,7 @@ g_dbus_menu_model_get_from_group (GDBusMenuGroup *group,
 /**
  * g_dbus_menu_model_get:
  * @connection: a #GDBusConnection
- * @bus_name: (nullable): the bus name which exports the menu model
- *     or %NULL if @connection is not a message bus connection
+ * @bus_name: the bus name which exports the menu model
  * @object_path: the object path at which the menu model is exported
  *
  * Obtains a #GDBusMenuModel for the menu model which is exported
@@ -900,8 +874,6 @@ g_dbus_menu_model_get (GDBusConnection *connection,
   GDBusMenuModel *proxy;
   GMainContext *context;
 
-  g_return_val_if_fail (bus_name != NULL || g_dbus_connection_get_unique_name (connection) == NULL, NULL);
-
   context = g_main_context_get_thread_default ();
   if (context == NULL)
     context = g_main_context_default ();
