@@ -27,11 +27,11 @@
 
 #include <gio/giotypes.h>
 
-#include <gio/gappinfo.h>
 #include <gio/gaction.h>
-#include <gio/gsimpleaction.h>
 #include <gio/gactiongroup.h>
-#include <gio/gsimpleactiongroup.h>
+#include <gio/gactiongroupexporter.h>
+#include <gio/gactionmap.h>
+#include <gio/gappinfo.h>
 #include <gio/gapplication.h>
 #include <gio/gapplicationcommandline.h>
 #include <gio/gasyncinitable.h>
@@ -106,6 +106,8 @@
 #include <gio/gseekable.h>
 #include <gio/gsettingsschema.h>
 #include <gio/gsettings.h>
+#include <gio/gsimpleaction.h>
+#include <gio/gsimpleactiongroup.h>
 #include <gio/gsimpleasyncresult.h>
 #include <gio/gsimplepermission.h>
 #include <gio/gsocketaddressenumerator.h>
@@ -144,6 +146,12 @@
 #include <gio/gdbusobjectmanager.h>
 #include <gio/gdbusobjectmanagerclient.h>
 #include <gio/gdbusobjectmanagerserver.h>
+#include <gio/gdbusactiongroup.h>
+#include <gio/gmenumodel.h>
+#include <gio/gmenu.h>
+#include <gio/gmenuexporter.h>
+#include <gio/gmenumarkup.h>
+#include <gio/gdbusmenumodel.h>
 
 #undef __GIO_GIO_H_INSIDE__
 
