@@ -1,12 +1,10 @@
 /*
  * Copyright © 2011 Canonical Ltd.
  *
- * SPDX-License-Identifier: LGPL-2.1-or-later
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation; either
- *  version 2.1 of the License, or (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU Lesser General Public License as
+ *  published by the Free Software Foundation; either version 2 of the
+ *  licence, or (at your option) any later version.
  *
  *  This library is distributed in the hope that it will be useful, but
  *  WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,13 +12,13 @@
  *  Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ *  License along with this library; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+ *  USA.
  *
  * Author: Ryan Lortie <desrt@desrt.ca>
  */
 
-#include "config.h"
-
 #include "gmenuexporter.h"
 
 #include "gdbusmethodinvocation.h"
@@ -28,19 +26,128 @@
 #include "gdbusnamewatching.h"
 #include "gdbuserror.h"
 
+/**
+ * SECTION:gmenuexporter
+ * @title: GMenuModel exporter
+ * @short_description: Export GMenuModels on D-Bus
+ * @see_also: #GMenuModel, #GMenuProxy
+ *
+ * These functions support exporting a #GMenuModel on D-Bus.
+ * The D-Bus interface that is used is a private implementation
+ * detail.
+ *
+ * To access an exported #GMenuModel remotely, use
+ * g_menu_proxy_get() to obtain a #GMenuProxy.
+ */
+
 /* {{{1 D-Bus Interface description */
 
-/* For documentation of this interface, see
- * https://wiki.gnome.org/Projects/GLib/GApplication/DBusAPI
+/* The org.gtk.Menus interface
+ * ===========================
+ *
+ * The interface is primarily concerned with three things:
+ *
+ * - communicating menus to the client
+ * - establishing links between menus and other menus
+ * - notifying clients of changes
+ *
+ * As a basic principle, it is recognised that the menu structure
+ * of an application is often large. It is also recognised that some
+ * menus are liable to frequently change without the user ever having
+ * opened the menu. For both of these reasons, the individual menus are
+ * arranged into subscription groups. Each subscription group is specified
+ * by an unsigned integer. The assignment of integers need not be consecutive.
+ *
+ * Within a subscription group there are multiple menus. Each menu is
+ * identified with an unsigned integer, unique to its subscription group.
+ *
+ * By convention, the primary menu is numbered 0 without subscription group 0.
+ *
+ * Actionable menu items (ie: those that produce some effect in the
+ * application when they are activated) have a related action, specified by
+ * a string. This string specifies the name of the action, according to the
+ * org.gtk.Actions interface, at the same object path as the menu.
+ *
+ * Methods
+ * -------
+ *
+ * Start :: (au) → (a(uuaa{sv}))
+ *
+ *   The Start method is used to indicate that a client is interested in
+ *   tracking and displaying the content of the menus of a particular list
+ *   of subscription groups.
+ *
+ *   Most typically, the client will request subscription group 0 to start.
+ *
+ *   The call has two effects. First, it replies with all menus defined
+ *   within the requested subscription groups. The format of the reply is
+ *   an array of tuples, where the items in each tuple are:
+ *   - the subscription group of the menu
+ *   - the number of the menu within that group
+ *   - an array of menu items
+ *
+ *   Each menu item is a dictionary of attributes (a{sv}).
+ *
+ *   Secondly, this call has a side effect: it atomically requests that
+ *   the Changed signal start to be emitted for the requested subscription
+ *   group. Each group has a subscription count and only signals changes
+ *   on itself when this count is greater than zero.
+ *
+ *   If a group is specified multiple times then the result is that the
+ *   contents of that group is only returned once, but the subscription
+ *   count is increased multiple times.
+ *
+ *   If a client disconnects from the bus while holding subscriptions then
+ *   its subscriptions will be cancelled. This prevents "leaking" subscriptions
+ *   in the case of crashes and is also useful for applications that want
+ *   to exit without manually cleaning up.
+ *
+ * End :: (au)
+ *
+ *   The End method reverses the previous effects of a call to Start.
+ *
+ *   When clients are no longer interested in the contents of a subscription
+ *   group, they should call the End method.
+ *
+ *   The parameter lists the subscription groups. A subscription group
+ *   needs to be cancelled the same number of times as it was requested.
+ *   For this reason, it might make sense to specify the same subscription
+ *   group multiple times (if multiple Start calls were made for this group).
+ *
+ * Signals
+ * -------
+ *
+ * Changed :: (a(uuuuaa{sv}))
+ *
+ *   The changed signal indicates changes to a particular menu.
+ *
+ *   The changes come as an array of tuples where the items in each tuple are:
+ *   - the subscription group of the menu
+ *   - the number of the menu within that group
+ *   - the position in the menu at which to make the change
+ *   - the number of items to delete from that position
+ *   - a list of new items to insert at that position
+ *
+ *   Each new menu item is a dictionary of attributes (a{sv}).
+ *
+ * Attributes
+ * ----------
+ *
+ * label (string): the label to display
+ * action (string): the name of the action
+ * target (variant): the parameter to pass when activating the action
+ * :section ((uu)): the menu to use to populate that section, specified
+ *     as a pair of subscription group and menu within that group
+ * :submenu ((uu)): the menu to use as a submenu, specified
+ *     as a pair of subscription group and menu within that group
  */
 
 static GDBusInterfaceInfo *
 org_gtk_Menus_get_interface (void)
 {
   static GDBusInterfaceInfo *interface_info;
-  static gsize interface_info_initialized = 0;
 
-  if (g_once_init_enter (&interface_info_initialized))
+  if (interface_info == NULL)
     {
       GError *error = NULL;
       GDBusNodeInfo *info;
@@ -60,13 +167,11 @@ org_gtk_Menus_get_interface (void)
                                            "  </interface>"
                                            "</node>", &error);
       if (info == NULL)
-        g_error ("%s", error->message);
+        g_error ("%s\n", error->message);
       interface_info = g_dbus_node_info_lookup_interface (info, "org.gtk.Menus");
       g_assert (interface_info != NULL);
       g_dbus_interface_info_ref (interface_info);
       g_dbus_node_info_unref (info);
-
-      g_once_init_leave (&interface_info_initialized, 1);
     }
 
   return interface_info;
@@ -197,7 +302,7 @@ g_menu_exporter_menu_describe_item (GMenuExporterMenu *menu,
   const char *name;
   GVariant *value;
 
-  g_variant_builder_init_static (&builder, G_VARIANT_TYPE_VARDICT);
+  g_variant_builder_init (&builder, G_VARIANT_TYPE_VARDICT);
 
   attr_iter = g_menu_model_iterate_item_attributes (menu->model, position);
   while (g_menu_attribute_iter_get_next (attr_iter, &name, &value))
@@ -221,7 +326,7 @@ g_menu_exporter_menu_list (GMenuExporterMenu *menu)
   GVariantBuilder builder;
   gint i, n;
 
-  g_variant_builder_init_static (&builder, G_VARIANT_TYPE ("aa{sv}"));
+  g_variant_builder_init (&builder, G_VARIANT_TYPE ("aa{sv}"));
 
   n = g_sequence_get_length (menu->item_links);
   for (i = 0; i < n; i++)
@@ -240,21 +345,10 @@ g_menu_exporter_menu_items_changed (GMenuModel *model,
   GMenuExporterMenu *menu = user_data;
   GSequenceIter *point;
   gint i;
-#ifndef G_DISABLE_ASSERT
-  gint n_items;
-#endif
 
   g_assert (menu->model == model);
   g_assert (menu->item_links != NULL);
-
-#ifndef G_DISABLE_ASSERT
-  n_items = g_sequence_get_length (menu->item_links);
-#endif
-  g_assert (position >= 0 && position < G_MENU_EXPORTER_MAX_SECTION_SIZE);
-  g_assert (removed >= 0 && removed < G_MENU_EXPORTER_MAX_SECTION_SIZE);
-  g_assert (added < G_MENU_EXPORTER_MAX_SECTION_SIZE);
-  g_assert (position + removed <= n_items);
-  g_assert (n_items - removed + added < G_MENU_EXPORTER_MAX_SECTION_SIZE);
+  g_assert (position + removed <= g_sequence_get_length (menu->item_links));
 
   point = g_sequence_get_iter_at_pos (menu->item_links, position + removed);
   g_sequence_remove_range (g_sequence_get_iter_at_pos (menu->item_links, position), point);
@@ -266,7 +360,7 @@ g_menu_exporter_menu_items_changed (GMenuModel *model,
     {
       GVariantBuilder builder;
 
-      g_variant_builder_init_static (&builder, G_VARIANT_TYPE ("(uuuuaa{sv})"));
+      g_variant_builder_init (&builder, G_VARIANT_TYPE ("(uuuuaa{sv})"));
       g_variant_builder_add (&builder, "u", g_menu_exporter_group_get_id (menu->group));
       g_variant_builder_add (&builder, "u", menu->id);
       g_variant_builder_add (&builder, "u", position);
@@ -359,15 +453,7 @@ g_menu_exporter_group_subscribe (GMenuExporterGroup *group,
       group->prepared = TRUE;
 
       menu = g_hash_table_lookup (group->menus, 0);
-
-      /* If the group was created by a subscription and does not yet
-       * exist, it won't have a root menu...
-       *
-       * That menu will be prepared if it is ever added (due to
-       * group->prepared == TRUE).
-       */
-      if (menu)
-        g_menu_exporter_menu_prepare (menu);
+      g_menu_exporter_menu_prepare (menu);
     }
 
   group->subscribed++;
@@ -378,7 +464,7 @@ g_menu_exporter_group_subscribe (GMenuExporterGroup *group,
       guint id = GPOINTER_TO_INT (key);
       GMenuExporterMenu *menu = val;
 
-      if (!g_sequence_is_empty (menu->item_links))
+      if (g_sequence_get_length (menu->item_links))
         {
           g_variant_builder_open (builder, G_VARIANT_TYPE ("(uuaa{sv})"));
           g_variant_builder_add (builder, "u", group->id);
@@ -478,7 +564,6 @@ g_menu_exporter_remote_subscribe (GMenuExporterRemote *remote,
   count = (gsize) g_hash_table_lookup (remote->watches, GINT_TO_POINTER (group_id));
   g_hash_table_insert (remote->watches, GINT_TO_POINTER (group_id), GINT_TO_POINTER (count + 1));
 
-  /* Group will be created (as empty/unsubscribed if it does not exist) */
   group = g_menu_exporter_lookup_group (remote->exporter, group_id);
   g_menu_exporter_group_subscribe (group, builder);
 }
@@ -526,9 +611,7 @@ g_menu_exporter_remote_free (gpointer data)
       g_menu_exporter_group_unsubscribe (group, GPOINTER_TO_INT (val));
     }
 
-  if (remote->watch_id > 0)
-    g_bus_unwatch_name (remote->watch_id);
-
+  g_bus_unwatch_name (remote->watch_id);
   g_hash_table_unref (remote->watches);
 
   g_slice_free (GMenuExporterRemote, remote);
@@ -559,7 +642,6 @@ struct _GMenuExporter
   guint next_group_id;
 
   GMenuExporterMenu *root;
-  GMenuExporterRemote *peer_remote;
   GHashTable *remotes;
 };
 
@@ -570,8 +652,7 @@ g_menu_exporter_name_vanished (GDBusConnection *connection,
 {
   GMenuExporter *exporter = user_data;
 
-  /* connection == NULL when we get called because the connection closed */
-  g_assert (exporter->connection == connection || connection == NULL);
+  g_assert (exporter->connection == connection);
 
   g_hash_table_remove (exporter->remotes, name);
 }
@@ -586,28 +667,19 @@ g_menu_exporter_subscribe (GMenuExporter *exporter,
   GVariantIter iter;
   guint32 id;
 
-  if (sender != NULL)
-    remote = g_hash_table_lookup (exporter->remotes, sender);
-  else
-    remote = exporter->peer_remote;
+  remote = g_hash_table_lookup (exporter->remotes, sender);
 
   if (remote == NULL)
     {
-      if (sender != NULL)
-        {
-          guint watch_id;
+      guint watch_id;
 
-          watch_id = g_bus_watch_name_on_connection (exporter->connection, sender, G_BUS_NAME_WATCHER_FLAGS_NONE,
-                                                     NULL, g_menu_exporter_name_vanished, exporter, NULL);
-          remote = g_menu_exporter_remote_new (exporter, watch_id);
-          g_hash_table_insert (exporter->remotes, g_strdup (sender), remote);
-        }
-      else
-        remote = exporter->peer_remote =
-          g_menu_exporter_remote_new (exporter, 0);
+      watch_id = g_bus_watch_name_on_connection (exporter->connection, sender, G_BUS_NAME_WATCHER_FLAGS_NONE,
+                                                 NULL, g_menu_exporter_name_vanished, exporter, NULL);
+      remote = g_menu_exporter_remote_new (exporter, watch_id);
+      g_hash_table_insert (exporter->remotes, g_strdup (sender), remote);
     }
 
-  g_variant_builder_init_static (&builder, G_VARIANT_TYPE ("(a(uuaa{sv}))"));
+  g_variant_builder_init (&builder, G_VARIANT_TYPE ("(a(uuaa{sv}))"));
 
   g_variant_builder_open (&builder, G_VARIANT_TYPE ("a(uuaa{sv})"));
 
@@ -629,10 +701,7 @@ g_menu_exporter_unsubscribe (GMenuExporter *exporter,
   GVariantIter iter;
   guint32 id;
 
-  if (sender != NULL)
-    remote = g_hash_table_lookup (exporter->remotes, sender);
-  else
-    remote = exporter->peer_remote;
+  remote = g_hash_table_lookup (exporter->remotes, sender);
 
   if (remote == NULL)
     return;
@@ -642,12 +711,7 @@ g_menu_exporter_unsubscribe (GMenuExporter *exporter,
     g_menu_exporter_remote_unsubscribe (remote, id);
 
   if (!g_menu_exporter_remote_has_subscriptions (remote))
-    {
-      if (sender != NULL)
-        g_hash_table_remove (exporter->remotes, sender);
-      else
-        g_clear_pointer (&exporter->peer_remote, g_menu_exporter_remote_free);
-    }
+    g_hash_table_remove (exporter->remotes, sender);
 }
 
 static void
@@ -656,7 +720,7 @@ g_menu_exporter_report (GMenuExporter *exporter,
 {
   GVariantBuilder builder;
 
-  g_variant_builder_init_static (&builder, G_VARIANT_TYPE_TUPLE);
+  g_variant_builder_init (&builder, G_VARIANT_TYPE_TUPLE);
   g_variant_builder_open (&builder, G_VARIANT_TYPE_ARRAY);
   g_variant_builder_add_value (&builder, report);
   g_variant_builder_close (&builder);
@@ -707,10 +771,11 @@ g_menu_exporter_create_group (GMenuExporter *exporter)
 }
 
 static void
-g_menu_exporter_free (GMenuExporter *exporter)
+g_menu_exporter_free (gpointer user_data)
 {
-  g_clear_pointer (&exporter->root, g_menu_exporter_menu_free);
-  g_clear_pointer (&exporter->peer_remote, g_menu_exporter_remote_free);
+  GMenuExporter *exporter = user_data;
+
+  g_menu_exporter_menu_free (exporter->root);
   g_hash_table_unref (exporter->remotes);
   g_hash_table_unref (exporter->groups);
   g_object_unref (exporter->connection);
@@ -763,14 +828,10 @@ g_menu_exporter_method_call (GDBusConnection       *connection,
  * The implemented D-Bus API should be considered private.
  * It is subject to change in the future.
  *
- * An object path can only have one menu model exported on it. If this
+ * An object path can only have one action group exported on it. If this
  * constraint is violated, the export will fail and 0 will be
  * returned (with @error set accordingly).
  *
- * Exporting menus with sections containing more than
- * %G_MENU_EXPORTER_MAX_SECTION_SIZE items is not supported and results in
- * undefined behavior.
- *
  * You can unexport the menu model using
  * g_dbus_connection_unexport_menu_model() with the return value of
  * this function.
@@ -786,22 +847,27 @@ g_dbus_connection_export_menu_model (GDBusConnection  *connection,
                                      GError          **error)
 {
   const GDBusInterfaceVTable vtable = {
-    g_menu_exporter_method_call, NULL, NULL, { 0 }
+    g_menu_exporter_method_call,
   };
   GMenuExporter *exporter;
   guint id;
 
   exporter = g_slice_new0 (GMenuExporter);
+
+  id = g_dbus_connection_register_object (connection, object_path, org_gtk_Menus_get_interface (),
+                                          &vtable, exporter, g_menu_exporter_free, error);
+
+  if (id == 0)
+    {
+      g_slice_free (GMenuExporter, exporter);
+      return 0;
+    }
+
   exporter->connection = g_object_ref (connection);
   exporter->object_path = g_strdup (object_path);
   exporter->groups = g_hash_table_new (NULL, NULL);
   exporter->remotes = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_menu_exporter_remote_free);
-
-  id = g_dbus_connection_register_object (connection, object_path, org_gtk_Menus_get_interface (),
-                                          &vtable, exporter, (GDestroyNotify) g_menu_exporter_free, error);
-
-  if (id != 0)
-    exporter->root = g_menu_exporter_group_add_menu (g_menu_exporter_create_group (exporter), menu);
+  exporter->root = g_menu_exporter_group_add_menu (g_menu_exporter_create_group (exporter), menu);
 
   return id;
 }
