@@ -1,10 +1,10 @@
 /*
  * Copyright © 2011 Canonical Ltd.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * licence, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,23 +12,19 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+ * USA.
  *
  * Author: Ryan Lortie <desrt@desrt.ca>
  */
 
-#include "config.h"
-
 #include "gmenumodel.h"
 
-#include "glibintl.h"
-#include "gmarshal-internal.h"
-
 /**
  * SECTION:gmenumodel
  * @title: GMenuModel
  * @short_description: An abstract class representing the contents of a menu
- * @include: gio/gio.h
  * @see_also: #GActionGroup
  *
  * #GMenuModel represents the contents of a menu -- an ordered list of
@@ -46,35 +42,39 @@
  * it (or, in the case of the 'root' menu, is defined by the context
  * in which it is used).
  *
- * As an example, consider the visible portions of this menu:
+ * As an example, consider the visible portions of the menu in
+ * <xref linkend="menu-example"/>.
  *
- * ## An example menu # {#menu-example}
- *
- * ![](menu-example.png)
+ * <figure id="menu-example">
+ *   <title>An example menu</title>
+ *   <graphic fileref="menu-example.png" format="PNG"></graphic>
+ * </figure>
  *
  * There are 8 "menus" visible in the screenshot: one menubar, two
  * submenus and 5 sections:
+ * <itemizedlist>
+ * <listitem>the toplevel menubar (containing 4 items)</listitem>
+ * <listitem>the View submenu (containing 3 sections)</listitem>
+ * <listitem>the first section of the View submenu (containing 2 items)</listitem>
+ * <listitem>the second section of the View submenu (containing 1 item)</listitem>
+ * <listitem>the final section of the View submenu (containing 1 item)</listitem>
+ * <listitem>the Highlight Mode submenu (containing 2 sections)</listitem>
+ * <listitem>the Sources section (containing 2 items)</listitem>
+ * <listitem>the Markup section (containing 2 items)</listitem>
+ * </itemizedlist>
  *
- * - the toplevel menubar (containing 4 items)
- * - the View submenu (containing 3 sections)
- * - the first section of the View submenu (containing 2 items)
- * - the second section of the View submenu (containing 1 item)
- * - the final section of the View submenu (containing 1 item)
- * - the Highlight Mode submenu (containing 2 sections)
- * - the Sources section (containing 2 items)
- * - the Markup section (containing 2 items)
- *
- * The [example][menu-model] illustrates the conceptual connection between
+ * <xref linkend="menu-model"/> illustrates the conceptual connection between
  * these 8 menus. Each large block in the figure represents a menu and the
  * smaller blocks within the large block represent items in that menu. Some
  * items contain references to other menus.
  *
- * ## A menu example # {#menu-model}
+ * <figure id="menu-model">
+ *   <title>A menu model</title>
+ *   <graphic fileref="menu-model.png" format="PNG"></graphic>
+ * </figure>
  *
- * ![](menu-model.png)
- *
- * Notice that the separators visible in the [example][menu-example]
- * appear nowhere in the [menu model][menu-model]. This is because
+ * Notice that the separators visible in <xref linkend="menu-example"/>
+ * appear nowhere in <xref linkend="menu-model"/>. This is because
  * separators are not explicitly represented in the menu model. Instead,
  * a separator is inserted between any two non-empty sections of a menu.
  * Section items can have labels just like any other item. In that case,
@@ -85,11 +85,13 @@
  * outside the application. Examples include global menus, jumplists,
  * dash boards, etc. To support such uses, it is necessary to 'export'
  * information about actions and their representation in menus, which
- * is exactly what the [GActionGroup exporter][gio-GActionGroup-exporter]
- * and the [GMenuModel exporter][gio-GMenuModel-exporter] do for
- * #GActionGroup and #GMenuModel. The client-side counterparts to
- * make use of the exported information are #GDBusActionGroup and
- * #GDBusMenuModel.
+ * is exactly what the
+ * <link linkend="gio-GActionGroup-exporter">GActionGroup exporter</link>
+ * and the
+ * <link linkend="gio-GMenuModel-exporter">GMenuModel exporter</link>
+ * do for #GActionGroup and #GMenuModel. The client-side counterparts
+ * to make use of the exported information are #GDBusActionGroup and
+ * #GMenuProxy.
  *
  * The API of #GMenuModel is very generic, with iterators for the
  * attributes and links of an item, see g_menu_model_iterate_item_attributes()
@@ -98,52 +100,53 @@
  * %G_MENU_ATTRIBUTE_ACTION, %G_MENU_ATTRIBUTE_TARGET, %G_MENU_LINK_SECTION
  * and %G_MENU_LINK_SUBMENU.
  *
- * Items in a #GMenuModel represent active controls if they refer to
- * an action that can get activated when the user interacts with the
- * menu item. The reference to the action is encoded by the string id
- * in the %G_MENU_ATTRIBUTE_ACTION attribute. An action id uniquely
- * identifies an action in an action group. Which action group(s) provide
- * actions depends on the context in which the menu model is used.
- * E.g. when the model is exported as the application menu of a
- * #GtkApplication, actions can be application-wide or window-specific
- * (and thus come from two different action groups). By convention, the
- * application-wide actions have names that start with "app.", while the
- * names of window-specific actions start with "win.".
+ * FIXME: explain how items are associated with actions.
  *
  * While a wide variety of stateful actions is possible, the following
  * is the minimum that is expected to be supported by all users of exported
  * menu information:
- * - an action with no parameter type and no state
- * - an action with no parameter type and boolean state
- * - an action with string parameter type and string state
- *
- * ## Stateless 
+ * <itemizedlist>
+ * <listitem>an action with no parameter type and no state</listitem>
+ * <listitem>an action with no parameter type and boolean state</listitem>
+ * <listitem>an action with string parameter type and string state</listitem>
+ * </itemizedlist>
  *
+ * <formalpara><title>Stateless</title>
+ * <para>
  * A stateless action typically corresponds to an ordinary menu item.
- *
+ * </para>
+ * <para>
  * Selecting such a menu item will activate the action (with no parameter).
+ * </para>
+ * </formalpara>
  *
- * ## Boolean State
- *
+ * <formalpara><title>Boolean State</title>
+ * <para>
  * An action with a boolean state will most typically be used with a "toggle"
  * or "switch" menu item. The state can be set directly, but activating the
  * action (with no parameter) results in the state being toggled.
- *
+ * </para>
+ * <para>
  * Selecting a toggle menu item will activate the action. The menu item should
  * be rendered as "checked" when the state is true.
+ * </para>
+ * </formalpara>
  *
- * ## String Parameter and State
- *
+ * <formalpara><title>String Parameter and State</title>
+ * <para>
  * Actions with string parameters and state will most typically be used to
  * represent an enumerated choice over the items available for a group of
  * radio menu items. Activating the action with a string parameter is
  * equivalent to setting that parameter as the state.
- *
+ * </para>
+ * <para>
  * Radio menu items, in addition to being associated with the action, will
  * have a target value. Selecting that menu item will result in activation
  * of the action with the target value as the parameter. The menu item should
  * be rendered as "selected" when the state of the action is equal to the
  * target value of the menu item.
+ * </para>
+ * </formalpara>
  */
 
 /**
@@ -182,8 +185,6 @@ typedef struct
 
 typedef GMenuLinkIterClass GMenuLinkHashIterClass;
 
-static GType g_menu_link_hash_iter_get_type (void);
-
 G_DEFINE_TYPE (GMenuLinkHashIter, g_menu_link_hash_iter, G_TYPE_MENU_LINK_ITER)
 
 static gboolean
@@ -238,8 +239,6 @@ typedef struct
 
 typedef GMenuAttributeIterClass GMenuAttributeHashIterClass;
 
-static GType g_menu_attribute_hash_iter_get_type (void);
-
 G_DEFINE_TYPE (GMenuAttributeHashIter, g_menu_attribute_hash_iter, G_TYPE_MENU_ATTRIBUTE_ITER)
 
 static gboolean
@@ -288,7 +287,7 @@ g_menu_attribute_hash_iter_class_init (GMenuAttributeHashIterClass *class)
 G_DEFINE_ABSTRACT_TYPE (GMenuModel, g_menu_model, G_TYPE_OBJECT)
 
 
-static guint g_menu_model_items_changed_signal;
+guint g_menu_model_items_changed_signal;
 
 static GMenuAttributeIter *
 g_menu_model_real_iterate_item_attributes (GMenuModel *model,
@@ -309,7 +308,7 @@ g_menu_model_real_iterate_item_attributes (GMenuModel *model,
   else
     {
       g_critical ("GMenuModel implementation '%s' doesn't override iterate_item_attributes() "
-                  "and fails to return valid values from get_item_attributes()",
+                  "and fails to return sane values from get_item_attributes()",
                   G_OBJECT_TYPE_NAME (model));
       result = NULL;
     }
@@ -373,7 +372,7 @@ g_menu_model_real_iterate_item_links (GMenuModel *model,
   else
     {
       g_critical ("GMenuModel implementation '%s' doesn't override iterate_item_links() "
-                  "and fails to return valid values from get_item_links()",
+                  "and fails to return sane values from get_item_links()",
                   G_OBJECT_TYPE_NAME (model));
       result = NULL;
     }
@@ -451,14 +450,10 @@ g_menu_model_class_init (GMenuModelClass *class)
    * reported.  The signal is emitted after the modification.
    **/
   g_menu_model_items_changed_signal =
-    g_signal_new (I_("items-changed"), G_TYPE_MENU_MODEL,
+    g_signal_new ("items-changed", G_TYPE_MENU_MODEL,
                   G_SIGNAL_RUN_LAST, 0, NULL, NULL,
-                  _g_cclosure_marshal_VOID__INT_INT_INT,
-                  G_TYPE_NONE,
+                  g_cclosure_marshal_generic, G_TYPE_NONE,
                   3, G_TYPE_INT, G_TYPE_INT, G_TYPE_INT);
-  g_signal_set_va_marshaller (g_menu_model_items_changed_signal,
-                              G_TYPE_FROM_CLASS (class),
-                              _g_cclosure_marshal_VOID__INT_INT_INTv);
 }
 
 /**
@@ -526,7 +521,7 @@ g_menu_model_iterate_item_attributes (GMenuModel *model,
  * @model: a #GMenuModel
  * @item_index: the index of the item
  * @attribute: the attribute to query
- * @expected_type: (nullable): the expected type of the attribute, or
+ * @expected_type: (allow-none): the expected type of the attribute, or
  *     %NULL
  *
  * Queries the item at position @item_index in @model for the attribute
@@ -574,12 +569,6 @@ g_menu_model_get_item_attribute_value (GMenuModel         *model,
  * type, then the positional parameters are ignored and %FALSE is
  * returned.
  *
- * This function is a mix of g_menu_model_get_item_attribute_value() and
- * g_variant_get(), followed by a g_variant_unref().  As such,
- * @format_string must make a complete copy of the data (since the
- * #GVariant may go away after the call to g_variant_unref()).  In
- * particular, no '&' characters are allowed in @format_string.
- *
  * Returns: %TRUE if the named attribute was found with the expected
  *     type
  *
@@ -592,20 +581,16 @@ g_menu_model_get_item_attribute (GMenuModel  *model,
                                  const gchar *format_string,
                                  ...)
 {
+  const GVariantType *expected_type;
   GVariant *value;
   va_list ap;
 
-  value = g_menu_model_get_item_attribute_value (model, item_index, attribute, NULL);
+  expected_type = NULL; /* XXX devine the type, ensure no '&' */
 
+  value = g_menu_model_get_item_attribute_value (model, item_index, attribute, expected_type);
   if (value == NULL)
     return FALSE;
 
-  if (!g_variant_check_format_string (value, format_string, TRUE))
-    {
-      g_variant_unref (value);
-      return FALSE;
-    }
-
   va_start (ap, format_string);
   g_variant_get_va (value, format_string, NULL, &ap);
   g_variant_unref (value);
@@ -695,6 +680,8 @@ g_menu_model_items_changed (GMenuModel *model,
   g_signal_emit (model, g_menu_model_items_changed_signal, 0, position, removed, added);
 }
 
+G_DEFINE_ABSTRACT_TYPE (GMenuAttributeIter, g_menu_attribute_iter, G_TYPE_OBJECT)
+
 struct _GMenuAttributeIterPrivate
 {
   GQuark name;
@@ -702,13 +689,11 @@ struct _GMenuAttributeIterPrivate
   gboolean valid;
 };
 
-G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GMenuAttributeIter, g_menu_attribute_iter, G_TYPE_OBJECT)
-
 /**
  * g_menu_attribute_iter_get_next:
  * @iter: a #GMenuAttributeIter
- * @out_name: (out) (optional) (transfer none): the type of the attribute
- * @value: (out) (optional) (transfer full): the attribute value
+ * @out_name: (out) (allow-none) (transfer none): the type of the attribute
+ * @value: (out) (allow-none) (transfer full): the attribute value
  *
  * This function combines g_menu_attribute_iter_next() with
  * g_menu_attribute_iter_get_name() and g_menu_attribute_iter_get_value().
@@ -719,8 +704,8 @@ G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GMenuAttributeIter, g_menu_attribute_iter,
  *
  * If successful, @name and @value are set to the name and value of the
  * attribute that has just been advanced to.  At this point,
- * g_menu_attribute_iter_get_name() and g_menu_attribute_iter_get_value() will
- * return the same values again.
+ * g_menu_item_get_name() and g_menu_item_get_value() will return the
+ * same values again.
  *
  * The value returned in @name remains valid for as long as the iterator
  * remains at the current position.  The value returned in @value must
@@ -840,7 +825,7 @@ g_menu_attribute_iter_finalize (GObject *object)
 static void
 g_menu_attribute_iter_init (GMenuAttributeIter *iter)
 {
-  iter->priv = g_menu_attribute_iter_get_instance_private (iter);
+  iter->priv = G_TYPE_INSTANCE_GET_PRIVATE (iter, G_TYPE_MENU_ATTRIBUTE_ITER, GMenuAttributeIterPrivate);
 }
 
 static void
@@ -849,8 +834,12 @@ g_menu_attribute_iter_class_init (GMenuAttributeIterClass *class)
   GObjectClass *object_class = G_OBJECT_CLASS (class);
 
   object_class->finalize = g_menu_attribute_iter_finalize;
+
+  g_type_class_add_private (class, sizeof (GMenuAttributeIterPrivate));
 }
 
+G_DEFINE_ABSTRACT_TYPE (GMenuLinkIter, g_menu_link_iter, G_TYPE_OBJECT)
+
 struct _GMenuLinkIterPrivate
 {
   GQuark name;
@@ -858,13 +847,11 @@ struct _GMenuLinkIterPrivate
   gboolean valid;
 };
 
-G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GMenuLinkIter, g_menu_link_iter, G_TYPE_OBJECT)
-
 /**
  * g_menu_link_iter_get_next:
  * @iter: a #GMenuLinkIter
- * @out_link: (out) (optional) (transfer none): the name of the link
- * @value: (out) (optional) (transfer full): the linked #GMenuModel
+ * @out_link: (out) (allow-none) (transfer none): the name of the link
+ * @value: (out) (allow-none) (transfer full): the linked #GMenuModel
  *
  * This function combines g_menu_link_iter_next() with
  * g_menu_link_iter_get_name() and g_menu_link_iter_get_value().
@@ -874,7 +861,7 @@ G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GMenuLinkIter, g_menu_link_iter, G_TYPE_OBJ
  *
  * If successful, @out_link and @value are set to the name and #GMenuModel
  * of the link that has just been advanced to.  At this point,
- * g_menu_link_iter_get_name() and g_menu_link_iter_get_value() will return the
+ * g_menu_item_get_name() and g_menu_item_get_value() will return the
  * same values again.
  *
  * The value returned in @out_link remains valid for as long as the iterator
@@ -994,7 +981,7 @@ g_menu_link_iter_finalize (GObject *object)
 static void
 g_menu_link_iter_init (GMenuLinkIter *iter)
 {
-  iter->priv = g_menu_link_iter_get_instance_private (iter);
+  iter->priv = G_TYPE_INSTANCE_GET_PRIVATE (iter, G_TYPE_MENU_LINK_ITER, GMenuLinkIterPrivate);
 }
 
 static void
@@ -1003,4 +990,6 @@ g_menu_link_iter_class_init (GMenuLinkIterClass *class)
   GObjectClass *object_class = G_OBJECT_CLASS (class);
 
   object_class->finalize = g_menu_link_iter_finalize;
+
+  g_type_class_add_private (class, sizeof (GMenuLinkIterPrivate));
 }
