@@ -2,12 +2,10 @@
  *
  * Copyright 2011 Red Hat, Inc.
  *
- * SPDX-License-Identifier: LGPL-2.1-or-later
- *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,14 +13,17 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * Public License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include "config.h"
 
 #include <errno.h>
-#include <string.h>
 #include <unistd.h>
+#include <linux/netlink.h>
+#include <linux/rtnetlink.h>
 
 #include "gnetworkmonitornetlink.h"
 #include "gcredentials.h"
@@ -30,47 +31,16 @@
 #include "ginitable.h"
 #include "giomodule-priv.h"
 #include "glibintl.h"
-#include "glib/gstdio.h"
 #include "gnetworkingprivate.h"
 #include "gnetworkmonitor.h"
 #include "gsocket.h"
 #include "gunixcredentialsmessage.h"
 
-/* must come at the end to pick system includes from
- * gnetworkingprivate.h */
-#ifdef HAVE_LINUX_NETLINK_H
-#include <linux/netlink.h>
-#include <linux/rtnetlink.h>
-#endif
-#if defined(HAVE_NETLINK_NETLINK_H) && defined(HAVE_NETLINK_NETLINK_ROUTE_H)
-#include <netlink/netlink.h>
-#include <netlink/netlink_route.h>
-#endif
-
-static GInitableIface *initable_parent_iface;
 static void g_network_monitor_netlink_iface_init (GNetworkMonitorInterface *iface);
 static void g_network_monitor_netlink_initable_iface_init (GInitableIface *iface);
 
-struct _GNetworkMonitorNetlinkPrivate
-{
-  GSocket *sock;
-  GSource *source, *dump_source;
-  GMainContext *context;
-
-  GPtrArray *dump_networks;
-};
-
-static gboolean read_netlink_messages (GNetworkMonitorNetlink  *nl,
-                                       GError                 **error);
-static gboolean read_netlink_messages_callback (GSocket             *socket,
-                                                GIOCondition         condition,
-                                                gpointer             user_data);
-static gboolean request_dump (GNetworkMonitorNetlink  *nl,
-                              GError                 **error);
-
 #define g_network_monitor_netlink_get_type _g_network_monitor_netlink_get_type
 G_DEFINE_TYPE_WITH_CODE (GNetworkMonitorNetlink, g_network_monitor_netlink, G_TYPE_NETWORK_MONITOR_BASE,
-                         G_ADD_PRIVATE (GNetworkMonitorNetlink)
                          G_IMPLEMENT_INTERFACE (G_TYPE_NETWORK_MONITOR,
                                                 g_network_monitor_netlink_iface_init)
                          G_IMPLEMENT_INTERFACE (G_TYPE_INITABLE,
@@ -81,31 +51,48 @@ G_DEFINE_TYPE_WITH_CODE (GNetworkMonitorNetlink, g_network_monitor_netlink, G_TY
                                                          "netlink",
                                                          20))
 
+struct _GNetworkMonitorNetlinkPrivate
+{
+  GSocket *sock;
+  GSource *source, *dump_source;
+
+  GPtrArray *dump_networks;
+};
+
+static gboolean read_netlink_messages (GSocket             *socket,
+                                       GIOCondition         condition,
+                                       gpointer             user_data);
+static gboolean request_dump (GNetworkMonitorNetlink  *nl,
+                              GError                 **error);
+
 static void
 g_network_monitor_netlink_init (GNetworkMonitorNetlink *nl)
 {
-  nl->priv = g_network_monitor_netlink_get_instance_private (nl);
+  nl->priv = G_TYPE_INSTANCE_GET_PRIVATE (nl,
+                                          G_TYPE_NETWORK_MONITOR_NETLINK,
+                                          GNetworkMonitorNetlinkPrivate);
 }
 
+
 static gboolean
 g_network_monitor_netlink_initable_init (GInitable     *initable,
                                          GCancellable  *cancellable,
                                          GError       **error)
 {
   GNetworkMonitorNetlink *nl = G_NETWORK_MONITOR_NETLINK (initable);
-  gint sockfd;
+  gint sockfd, val;
   struct sockaddr_nl snl;
 
   /* We create the socket the old-school way because sockaddr_netlink
    * can't be represented as a GSocketAddress
    */
-  sockfd = g_socket (PF_NETLINK, SOCK_RAW, NETLINK_ROUTE, NULL);
+  sockfd = socket (PF_NETLINK, SOCK_RAW, NETLINK_ROUTE);
   if (sockfd == -1)
     {
       int errsv = errno;
       g_set_error (error, G_IO_ERROR, g_io_error_from_errno (errsv),
                    _("Could not create network monitor: %s"),
-                   g_strerror (errsv));
+                   g_strerror (errno));
       return FALSE;
     }
 
@@ -117,30 +104,29 @@ g_network_monitor_netlink_initable_init (GInitable     *initable,
       int errsv = errno;
       g_set_error (error, G_IO_ERROR, g_io_error_from_errno (errsv),
                    _("Could not create network monitor: %s"),
-                   g_strerror (errsv));
-      (void) g_close (sockfd, NULL);
+                   g_strerror (errno));
+      close (sockfd);
       return FALSE;
     }
 
-  nl->priv->sock = g_socket_new_from_fd (sockfd, error);
-  if (!nl->priv->sock)
-    {
-      g_prefix_error (error, "%s", _("Could not create network monitor: "));
-      (void) g_close (sockfd, NULL);
-      return FALSE;
-    }
-
-#ifdef SO_PASSCRED
-  if (!g_socket_set_option (nl->priv->sock, SOL_SOCKET, SO_PASSCRED,
-			    TRUE, NULL))
+  val = 1;
+  if (setsockopt (sockfd, SOL_SOCKET, SO_PASSCRED, &val, sizeof (val)) != 0)
     {
       int errsv = errno;
       g_set_error (error, G_IO_ERROR, g_io_error_from_errno (errsv),
                    _("Could not create network monitor: %s"),
-                   g_strerror (errsv));
+                   g_strerror (errno));
+      close (sockfd);
+      return FALSE;
+    }
+
+  nl->priv->sock = g_socket_new_from_fd (sockfd, error);
+  if (error)
+    {
+      g_prefix_error (error, "%s", _("Could not create network monitor: "));
+      close (sockfd);
       return FALSE;
     }
-#endif
 
   /* Request the current state */
   if (!request_dump (nl, error))
@@ -151,23 +137,18 @@ g_network_monitor_netlink_initable_init (GInitable     *initable,
    */
   while (nl->priv->dump_networks)
     {
-      GError *local_error = NULL;
-      if (!read_netlink_messages (nl, &local_error))
-        {
-          g_warning ("%s", local_error->message);
-          g_clear_error (&local_error);
-          break;
-        }
+      if (!read_netlink_messages (NULL, G_IO_IN, nl))
+        break;
     }
 
   g_socket_set_blocking (nl->priv->sock, FALSE);
-  nl->priv->context = g_main_context_ref_thread_default ();
   nl->priv->source = g_socket_create_source (nl->priv->sock, G_IO_IN, NULL);
   g_source_set_callback (nl->priv->source,
-                         (GSourceFunc) read_netlink_messages_callback, nl, NULL);
-  g_source_attach (nl->priv->source, nl->priv->context);
+                         (GSourceFunc) read_netlink_messages, nl, NULL);
+  g_source_attach (nl->priv->source,
+                   g_main_context_get_thread_default ());
 
-  return initable_parent_iface->init (initable, cancellable, error);
+  return TRUE;
 }
 
 static gboolean
@@ -224,16 +205,19 @@ queue_request_dump (GNetworkMonitorNetlink *nl)
       g_source_unref (nl->priv->dump_source);
     }
 
-  nl->priv->dump_source = g_timeout_source_new_seconds (1);
+  nl->priv->dump_source = g_timeout_source_new (1000);
   g_source_set_callback (nl->priv->dump_source,
                          (GSourceFunc) timeout_request_dump, nl, NULL);
-  g_source_attach (nl->priv->dump_source, nl->priv->context);
+  g_source_attach (nl->priv->dump_source,
+                   g_main_context_get_thread_default ());
 }
 
-static GInetAddressMask *
-create_inet_address_mask (GSocketFamily  family,
-                          const guint8  *dest,
-                          gsize          dest_len)
+static void
+add_network (GNetworkMonitorNetlink *nl,
+             GSocketFamily           family,
+             gint                    dest_len,
+             guint8                 *dest,
+             guint8                 *gateway)
 {
   GInetAddress *dest_addr;
   GInetAddressMask *network;
@@ -244,53 +228,52 @@ create_inet_address_mask (GSocketFamily  family,
     dest_addr = g_inet_address_new_any (family);
   network = g_inet_address_mask_new (dest_addr, dest_len, NULL);
   g_object_unref (dest_addr);
-
-  return network;
-}
-
-static void
-add_network (GNetworkMonitorNetlink *nl,
-             GSocketFamily           family,
-             const guint8           *dest,
-             gsize                   dest_len)
-{
-  GInetAddressMask *network = create_inet_address_mask (family, dest, dest_len);
   g_return_if_fail (network != NULL);
 
   if (nl->priv->dump_networks)
-    g_ptr_array_add (nl->priv->dump_networks, g_object_ref (network));
+    g_ptr_array_add (nl->priv->dump_networks, network);
   else
-    g_network_monitor_base_add_network (G_NETWORK_MONITOR_BASE (nl), network);
-
-  g_object_unref (network);
+    {
+      g_network_monitor_base_add_network (G_NETWORK_MONITOR_BASE (nl), network);
+      g_object_unref (network);
+    }
 }
 
 static void
 remove_network (GNetworkMonitorNetlink *nl,
                 GSocketFamily           family,
-                const guint8           *dest,
-                gsize                   dest_len)
+                gint                    dest_len,
+                guint8                 *dest,
+                guint8                 *gateway)
 {
-  GInetAddressMask *network = create_inet_address_mask (family, dest, dest_len);
+  GInetAddress *dest_addr;
+  GInetAddressMask *network;
+
+  if (dest)
+    dest_addr = g_inet_address_new_from_bytes (dest, family);
+  else
+    dest_addr = g_inet_address_new_any (family);
+  network = g_inet_address_mask_new (dest_addr, dest_len, NULL);
+  g_object_unref (dest_addr);
   g_return_if_fail (network != NULL);
 
   if (nl->priv->dump_networks)
     {
       GInetAddressMask **dump_networks = (GInetAddressMask **)nl->priv->dump_networks->pdata;
-      guint i;
+      int i;
 
       for (i = 0; i < nl->priv->dump_networks->len; i++)
         {
           if (g_inet_address_mask_equal (network, dump_networks[i]))
             g_ptr_array_remove_index_fast (nl->priv->dump_networks, i--);
         }
+      g_object_unref (network);
     }
   else
     {
       g_network_monitor_base_remove_network (G_NETWORK_MONITOR_BASE (nl), network);
+      g_object_unref (network);
     }
-
-  g_object_unref (network);
 }
 
 static void
@@ -299,47 +282,64 @@ finish_dump (GNetworkMonitorNetlink *nl)
   g_network_monitor_base_set_networks (G_NETWORK_MONITOR_BASE (nl),
                                        (GInetAddressMask **)nl->priv->dump_networks->pdata,
                                        nl->priv->dump_networks->len);
-  g_ptr_array_free (nl->priv->dump_networks, TRUE);
+  g_ptr_array_free (nl->priv->dump_networks, FALSE);
   nl->priv->dump_networks = NULL;
 }
 
 static gboolean
-read_netlink_messages (GNetworkMonitorNetlink  *nl,
-                       GError                 **error)
+read_netlink_messages (GSocket      *socket,
+                       GIOCondition  condition,
+                       gpointer      user_data)
 {
+  GNetworkMonitorNetlink *nl = user_data;
   GInputVector iv;
-  gssize len;
-  gint flags;
-  GError *local_error = NULL;
-  GSocketAddress *addr = NULL;
+  gsize len;
+  GSocketControlMessage **cmsgs = NULL;
+  gint num_cmsgs = 0, i, flags;
+  GError *error = NULL;
+  GCredentials *creds;
+  uid_t sender;
   struct nlmsghdr *msg;
   struct rtmsg *rtmsg;
   struct rtattr *attr;
-  struct sockaddr_nl source_sockaddr;
   gsize attrlen;
-  guint8 *dest, *gateway, *oif;
+  guint8 *dest, *gateway;
+  gboolean retval = TRUE;
 
   iv.buffer = NULL;
   iv.size = 0;
 
   flags = MSG_PEEK | MSG_TRUNC;
   len = g_socket_receive_message (nl->priv->sock, NULL, &iv, 1,
-                                  NULL, NULL, &flags, NULL, &local_error);
+                                  NULL, NULL, &flags, NULL, &error);
   if (len < 0)
-    goto done;
+    {
+      g_warning ("Error on netlink socket: %s", error->message);
+      g_error_free (error);
+      if (nl->priv->dump_networks)
+        finish_dump (nl);
+      return FALSE;
+    }
 
   iv.buffer = g_malloc (len);
   iv.size = len;
-  len = g_socket_receive_message (nl->priv->sock, &addr, &iv, 1,
-                                  NULL, NULL, NULL, NULL, &local_error);
+  len = g_socket_receive_message (nl->priv->sock, NULL, &iv, 1,
+                                  &cmsgs, &num_cmsgs, NULL, NULL, &error);
   if (len < 0)
+    {
+      g_warning ("Error on netlink socket: %s", error->message);
+      g_error_free (error);
+      if (nl->priv->dump_networks)
+        finish_dump (nl);
+      return FALSE;
+    }
+
+  if (num_cmsgs != 1 || !G_IS_UNIX_CREDENTIALS_MESSAGE (cmsgs[0]))
     goto done;
 
-  if (!g_socket_address_to_native (addr, &source_sockaddr, sizeof (source_sockaddr), &local_error))
-    goto done;
-
-  /* If the sender port id is 0 (not fakeable) then the message is from the kernel */
-  if (source_sockaddr.nl_pid != 0)
+  creds = g_unix_credentials_message_get_credentials (G_UNIX_CREDENTIALS_MESSAGE (cmsgs[0]));
+  sender = g_credentials_get_unix_user (creds, NULL);
+  if (sender != 0)
     goto done;
 
   msg = (struct nlmsghdr *) iv.buffer;
@@ -347,10 +347,8 @@ read_netlink_messages (GNetworkMonitorNetlink  *nl,
     {
       if (!NLMSG_OK (msg, (size_t) len))
         {
-          g_set_error_literal (&local_error,
-                               G_IO_ERROR,
-                               G_IO_ERROR_PARTIAL_INPUT,
-                               "netlink message was truncated; shouldn't happen...");
+          g_warning ("netlink message was truncated; shouldn't happen...");
+          retval = FALSE;
           goto done;
         }
 
@@ -367,37 +365,22 @@ read_netlink_messages (GNetworkMonitorNetlink  *nl,
 
           attrlen = NLMSG_PAYLOAD (msg, sizeof (struct rtmsg));
           attr = RTM_RTA (rtmsg);
-          dest = gateway = oif = NULL;
+          dest = gateway = NULL;
           while (RTA_OK (attr, attrlen))
             {
               if (attr->rta_type == RTA_DST)
                 dest = RTA_DATA (attr);
               else if (attr->rta_type == RTA_GATEWAY)
                 gateway = RTA_DATA (attr);
-              else if (attr->rta_type == RTA_OIF)
-                oif = RTA_DATA (attr);
               attr = RTA_NEXT (attr, attrlen);
             }
 
-          if (dest || gateway || oif)
+          if (dest || gateway)
             {
-              /* Unless we're processing the results of a dump, ignore
-               * IPv6 link-local multicast routes, which are added and
-               * removed all the time for some reason.
-               */
-#define UNALIGNED_IN6_IS_ADDR_MC_LINKLOCAL(a)           \
-              ((a[0] == 0xff) && ((a[1] & 0xf) == 0x2))
-
-              if (!nl->priv->dump_networks &&
-                  rtmsg->rtm_family == AF_INET6 &&
-                  rtmsg->rtm_dst_len != 0 &&
-                  (dest && UNALIGNED_IN6_IS_ADDR_MC_LINKLOCAL (dest)))
-                continue;
-
               if (msg->nlmsg_type == RTM_NEWROUTE)
-                add_network (nl, rtmsg->rtm_family, dest, rtmsg->rtm_dst_len);
+                add_network (nl, rtmsg->rtm_family, rtmsg->rtm_dst_len, dest, gateway);
               else
-                remove_network (nl, rtmsg->rtm_family, dest, rtmsg->rtm_dst_len);
+                remove_network (nl, rtmsg->rtm_family, rtmsg->rtm_dst_len, dest, gateway);
               queue_request_dump (nl);
             }
           break;
@@ -410,40 +393,28 @@ read_netlink_messages (GNetworkMonitorNetlink  *nl,
           {
             struct nlmsgerr *e = NLMSG_DATA (msg);
 
-            g_set_error (&local_error,
-                         G_IO_ERROR,
-                         g_io_error_from_errno (-e->error),
-                         "netlink error: %s",
-                         g_strerror (-e->error));
+            g_warning ("netlink error: %s", g_strerror (-e->error));
           }
+          retval = FALSE;
           goto done;
 
         default:
-          g_set_error (&local_error,
-                       G_IO_ERROR,
-                       G_IO_ERROR_INVALID_DATA,
-                       "unexpected netlink message %d",
-                       msg->nlmsg_type);
+          g_warning ("unexpected netlink message %d", msg->nlmsg_type);
+          retval = FALSE;
           goto done;
         }
     }
 
  done:
+  for (i = 0; i < num_cmsgs; i++)
+    g_object_unref (cmsgs[i]);
+  g_free (cmsgs);
+
   g_free (iv.buffer);
-  g_clear_object (&addr);
 
-  if (local_error != NULL && nl->priv->dump_networks)
+  if (!retval && nl->priv->dump_networks)
     finish_dump (nl);
-
-  if (local_error != NULL)
-    {
-      g_propagate_prefixed_error (error, local_error, "Error on netlink socket: ");
-      return FALSE;
-    }
-  else
-    {
-      return TRUE;
-    }
+  return retval;
 }
 
 static void
@@ -451,6 +422,12 @@ g_network_monitor_netlink_finalize (GObject *object)
 {
   GNetworkMonitorNetlink *nl = G_NETWORK_MONITOR_NETLINK (object);
 
+  if (nl->priv->sock)
+    {
+      g_socket_close (nl->priv->sock, NULL);
+      g_object_unref (nl->priv->sock);
+    }
+
   if (nl->priv->source)
     {
       g_source_destroy (nl->priv->source);
@@ -463,42 +440,17 @@ g_network_monitor_netlink_finalize (GObject *object)
       g_source_unref (nl->priv->dump_source);
     }
 
-  if (nl->priv->sock)
-    {
-      g_socket_close (nl->priv->sock, NULL);
-      g_object_unref (nl->priv->sock);
-    }
-
-  g_clear_pointer (&nl->priv->context, g_main_context_unref);
-  g_clear_pointer (&nl->priv->dump_networks, g_ptr_array_unref);
-
   G_OBJECT_CLASS (g_network_monitor_netlink_parent_class)->finalize (object);
 }
 
-static gboolean
-read_netlink_messages_callback (GSocket      *socket,
-                                GIOCondition  condition,
-                                gpointer      user_data)
-{
-  GError *error = NULL;
-  GNetworkMonitorNetlink *nl = G_NETWORK_MONITOR_NETLINK (user_data);
-
-  if (!read_netlink_messages (nl, &error))
-    {
-      g_warning ("Error reading netlink message: %s", error->message);
-      g_clear_error (&error);
-      return FALSE;
-    }
-
-  return TRUE;
-}
-
 static void
 g_network_monitor_netlink_class_init (GNetworkMonitorNetlinkClass *nl_class)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (nl_class);
 
-  gobject_class->finalize = g_network_monitor_netlink_finalize;
+  g_type_class_add_private (nl_class, sizeof (GNetworkMonitorNetlinkPrivate));
+
+  gobject_class->finalize  = g_network_monitor_netlink_finalize;
 }
 
 static void
@@ -509,7 +461,5 @@ g_network_monitor_netlink_iface_init (GNetworkMonitorInterface *monitor_iface)
 static void
 g_network_monitor_netlink_initable_iface_init (GInitableIface *iface)
 {
-  initable_parent_iface = g_type_interface_peek_parent (iface);
-
   iface->init = g_network_monitor_netlink_initable_init;
 }
