@@ -35,7 +35,10 @@
 
 #include "garray.h"
 
+#include "gbytes.h"
+#include "gslice.h"
 #include "gmem.h"
+#include "gtestutils.h"
 #include "gthread.h"
 #include "gmessages.h"
 #include "gqsort.h"
@@ -223,14 +226,6 @@ g_array_ref (GArray *array)
   return array;
 }
 
-typedef enum
-{
-  FREE_SEGMENT = 1 << 0,
-  PRESERVE_WRAPPER = 1 << 1
-} ArrayFreeFlags;
-
-static gchar *array_free (GRealArray *, ArrayFreeFlags);
-
 /**
  * g_array_unref:
  * @array: A #GArray.
@@ -249,7 +244,7 @@ g_array_unref (GArray *array)
   g_return_if_fail (array);
 
   if (g_atomic_int_dec_and_test (&rarray->ref_count))
-    array_free (rarray, FREE_SEGMENT);
+    g_array_free (array, TRUE);
 }
 
 /**
@@ -295,26 +290,17 @@ g_array_free (GArray   *farray,
 	      gboolean  free_segment)
 {
   GRealArray *array = (GRealArray*) farray;
-  ArrayFreeFlags flags;
+  gchar* segment;
+  gboolean preserve_wrapper;
 
   g_return_val_if_fail (array, NULL);
 
-  flags = (free_segment ? FREE_SEGMENT : 0);
-
   /* if others are holding a reference, preserve the wrapper but do free/return the data */
-  if (!g_atomic_int_dec_and_test (&array->ref_count))
-    flags |= PRESERVE_WRAPPER;
+  preserve_wrapper = FALSE;
+  if (g_atomic_int_get (&array->ref_count) > 1)
+    preserve_wrapper = TRUE;
 
-  return array_free (array, flags);
-}
-
-static gchar *
-array_free (GRealArray     *array,
-            ArrayFreeFlags  flags)
-{
-  gchar *segment;
-
-  if (flags & FREE_SEGMENT)
+  if (free_segment)
     {
       g_free (array->data);
       segment = NULL;
@@ -322,7 +308,7 @@ array_free (GRealArray     *array,
   else
     segment = (gchar*) array->data;
 
-  if (flags & PRESERVE_WRAPPER)
+  if (preserve_wrapper)
     {
       array->data            = NULL;
       array->len             = 0;
@@ -926,8 +912,6 @@ g_ptr_array_ref (GPtrArray *array)
   return array;
 }
 
-static gpointer *ptr_array_free (GPtrArray *, ArrayFreeFlags);
-
 /**
  * g_ptr_array_unref:
  * @array: A #GPtrArray.
@@ -946,7 +930,7 @@ g_ptr_array_unref (GPtrArray *array)
   g_return_if_fail (array);
 
   if (g_atomic_int_dec_and_test (&rarray->ref_count))
-    ptr_array_free (array, FREE_SEGMENT);
+    g_ptr_array_free (array, TRUE);
 }
 
 /**
@@ -972,27 +956,17 @@ g_ptr_array_free (GPtrArray *farray,
 		  gboolean   free_segment)
 {
   GRealPtrArray *array = (GRealPtrArray*) farray;
-  ArrayFreeFlags flags;
+  gpointer* segment;
+  gboolean preserve_wrapper;
 
   g_return_val_if_fail (array, NULL);
 
-  flags = (free_segment ? FREE_SEGMENT : 0);
-
   /* if others are holding a reference, preserve the wrapper but do free/return the data */
-  if (!g_atomic_int_dec_and_test (&array->ref_count))
-    flags |= PRESERVE_WRAPPER;
+  preserve_wrapper = FALSE;
+  if (g_atomic_int_get (&array->ref_count) > 1)
+    preserve_wrapper = TRUE;
 
-  return ptr_array_free (farray, flags);
-}
-
-static gpointer *
-ptr_array_free (GPtrArray      *farray,
-                ArrayFreeFlags  flags)
-{
-  GRealPtrArray *array = (GRealPtrArray*) farray;
-  gpointer *segment;
-
-  if (flags & FREE_SEGMENT)
+  if (free_segment)
     {
       if (array->element_free_func != NULL)
         g_ptr_array_foreach (farray, (GFunc) array->element_free_func, NULL);
@@ -1002,7 +976,7 @@ ptr_array_free (GPtrArray      *farray,
   else
     segment = array->pdata;
 
-  if (flags & PRESERVE_WRAPPER)
+  if (preserve_wrapper)
     {
       array->pdata = NULL;
       array->len = 0;
@@ -1372,16 +1346,13 @@ g_ptr_array_foreach (GPtrArray *array,
 /**
  * SECTION:arrays_byte
  * @title: Byte Arrays
- * @short_description: arrays of bytes, which grow automatically as
- *                     elements are added
+ * @short_description: arrays of bytes
  *
- * #GByteArray is based on #GArray, to provide arrays of bytes which
- * grow automatically as elements are added.
+ * #GByteArray is a mutable array of bytes based on #GArray, to provide arrays
+ * of bytes which grow automatically as elements are added.
  *
- * To create a new #GByteArray use g_byte_array_new().
- *
- * To add elements to a #GByteArray, use g_byte_array_append(), and
- * g_byte_array_prepend().
+ * To create a new #GByteArray use g_byte_array_new(). To add elements to a
+ * #GByteArray, use g_byte_array_append(), and g_byte_array_prepend().
  *
  * To set the size of a #GByteArray, use g_byte_array_set_size().
  *
@@ -1408,6 +1379,9 @@ g_ptr_array_foreach (GPtrArray *array,
  *   g_byte_array_free (gbarray, TRUE);
  *  </programlisting>
  * </example>
+ *
+ * See #GBytes if you are interested in an immutable object representing a
+ * sequence of bytes.
  **/
 
 /**
@@ -1431,6 +1405,36 @@ GByteArray* g_byte_array_new (void)
   return (GByteArray*) g_array_sized_new (FALSE, FALSE, 1, 0);
 }
 
+/**
+ * g_byte_array_new_take:
+ * @data: (array length=len): byte data for the array
+ * @len: length of @data
+ *
+ * Create byte array containing the data. The data will be owned by the array
+ * and will be freed with g_free(), i.e. it could be allocated using g_strdup().
+ *
+ * Since: 2.32
+ *
+ * Returns: (transfer full): a new #GByteArray
+ */
+GByteArray *
+g_byte_array_new_take (guint8 *data,
+                       gsize   len)
+{
+  GByteArray *array;
+  GRealArray *real;
+
+  array = g_byte_array_new ();
+  real = (GRealArray *)array;
+  g_assert (real->data == NULL);
+  g_assert (real->len == 0);
+
+  real->data = data;
+  real->len = len;
+
+  return array;
+}
+
 /**
  * g_byte_array_sized_new:
  * @reserved_size: number of bytes preallocated.
@@ -1464,6 +1468,35 @@ guint8*	    g_byte_array_free     (GByteArray *array,
   return (guint8*) g_array_free ((GArray*) array, free_segment);
 }
 
+/**
+ * g_byte_array_free_to_bytes:
+ * @array: (transfer full): a #GByteArray
+ *
+ * Transfers the data from the #GByteArray into a new immutable #GBytes.
+ *
+ * The #GByteArray is freed unless the reference count of @array is greater
+ * than one, the #GByteArray wrapper is preserved but the size of @array
+ * will be set to zero.
+ *
+ * This is identical to using g_bytes_new_take() and g_byte_array_free()
+ * together.
+ *
+ * Since: 2.32
+ *
+ * Returns: (transfer full): a new immutable #GBytes representing same byte
+ *          data that was in the array
+ */
+GBytes *
+g_byte_array_free_to_bytes (GByteArray *array)
+{
+  gsize length;
+
+  g_return_val_if_fail (array != NULL, NULL);
+
+  length = array->len;
+  return g_bytes_new_take (g_byte_array_free (array, FALSE), length);
+}
+
 /**
  * g_byte_array_ref:
  * @array: A #GByteArray.
